@@ -1,6 +1,6 @@
 /* gpgme.c -  GnuPG Made Easy
  *	Copyright (C) 2000 Werner Koch (dd9jn)
- *      Copyright (C) 2001 g10 Code GmbH
+ *      Copyright (C) 2001, 2002 g10 Code GmbH
  *
  * This file is part of GPGME.
  *
@@ -22,132 +22,124 @@
 #include <config.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <assert.h>
 
 #include "util.h"
 #include "context.h"
 #include "ops.h"
 
-#define my_isdigit(a)  ( (a) >='0' && (a) <= '9' )
-#define my_isxdigit(a) ( my_isdigit((a))               \
-                         || ((a) >= 'A' && (a) <= 'F') \
-                         || ((a) >= 'f' && (a) <= 'f') )
 
 /**
  * gpgme_new:
  * @r_ctx: Returns the new context
- * 
- * Create a new context to be used with most of the other GPGME
- * functions.  Use gpgme_release_contect() to release all resources
  *
- * Return value: An error code 
+ * Create a new context to be used with most of the other GPGME
+ * functions.  Use gpgme_release_context() to release all resources
+ *
+ * Return value: An error code
  **/
 GpgmeError
 gpgme_new (GpgmeCtx *r_ctx)
 {
-    GpgmeCtx c;
+  GpgmeCtx ctx;
 
-    c = xtrycalloc ( 1, sizeof *c );
-    if (!c)
-        return mk_error (Out_Of_Core);
-    c->verbosity = 1;
-    *r_ctx = c;
+  if (!r_ctx)
+    return mk_error (Invalid_Value);
+  *r_ctx = 0;
+  ctx = xtrycalloc (1, sizeof *ctx);
+  if (!ctx)
+    return mk_error (Out_Of_Core);
+  ctx->keylist_mode = GPGME_KEYLIST_MODE_LOCAL;
+  ctx->verbosity = 1;
+  ctx->include_certs = 1;
+  *r_ctx = ctx;
 
-    return 0;
+  return 0;
 }
 
+
 /**
  * gpgme_release:
- * @c: Context to be released. 
- * 
+ * @c: Context to be released.
+ *
  * Release all resources associated with the given context.
  **/
 void
-gpgme_release ( GpgmeCtx c )
+gpgme_release (GpgmeCtx ctx)
 {
-    if (!c)
-        return;
-    _gpgme_gpg_release ( c->gpg ); 
-    _gpgme_release_result ( c );
-    gpgme_key_release ( c->tmp_key );
-    gpgme_data_release ( c->help_data_1 );
-    gpgme_data_release ( c->notation );
-    gpgme_signers_clear (c);
-    if (c->signers)
-        xfree (c->signers);
-    /* fixme: release the key_queue */
-    xfree (c);
+  if (!ctx)
+    return;
+  _gpgme_engine_release (ctx->engine);
+  _gpgme_release_result (ctx);
+  gpgme_key_release (ctx->tmp_key);
+  gpgme_data_release (ctx->help_data_1);
+  gpgme_data_release (ctx->notation);
+  gpgme_signers_clear (ctx);
+  if (ctx->signers)
+    xfree (ctx->signers);
+  /* FIXME: Release the key_queue.  */
+  xfree (ctx);
 }
 
-
 void
-_gpgme_release_result ( GpgmeCtx c )
+_gpgme_release_result (GpgmeCtx ctx)
 {
-    switch (c->result_type) {
-      case RESULT_TYPE_NONE:
-        break;
-      case RESULT_TYPE_VERIFY:
-        _gpgme_release_verify_result ( c->result.verify );
-        break;
-      case RESULT_TYPE_DECRYPT:
-        _gpgme_release_decrypt_result ( c->result.decrypt );
-        break;
-      case RESULT_TYPE_SIGN:
-        _gpgme_release_sign_result ( c->result.sign );
-        break;
-      case RESULT_TYPE_ENCRYPT:
-        _gpgme_release_encrypt_result ( c->result.encrypt );
-        break;
-    }
-
-    c->result.verify = NULL;
-    c->result_type = RESULT_TYPE_NONE;
-    _gpgme_set_op_info (c, NULL);
+  _gpgme_release_verify_result (ctx->result.verify);
+  _gpgme_release_decrypt_result (ctx->result.decrypt);
+  _gpgme_release_sign_result (ctx->result.sign);
+  _gpgme_release_encrypt_result (ctx->result.encrypt);
+  _gpgme_release_passphrase_result (ctx->result.passphrase);
+  _gpgme_release_import_result (ctx->result.import);
+  _gpgme_release_delete_result (ctx->result.delete);
+  _gpgme_release_genkey_result (ctx->result.genkey);
+  memset (&ctx->result, 0, sizeof (ctx->result));
+  _gpgme_set_op_info (ctx, NULL);
+  ctx->error = 0;
 }
 
 
 /**
  * gpgme_cancel:
  * @c: the context
- * 
+ *
  * Cancel the current operation.  It is not guaranteed that it will work for
  * all kinds of operations.  It is especially useful in a passphrase callback
  * to stop the system from asking another time for the passphrase.
  **/
-
 void
-gpgme_cancel (GpgmeCtx c)
+gpgme_cancel (GpgmeCtx ctx)
 {
-    return_if_fail (c);
+  return_if_fail (ctx);
 
-    c->cancel = 1;
+  ctx->cancel = 1;
 }
 
 /**
  * gpgme_get_notation:
- * @c: the context 
- * 
+ * @c: the context
+ *
  * If there is notation data available from the last signature check,
  * this function may be used to return this notation data as a string.
  * The string is an XML represantaton of that data embedded in a
  * %&lt;notation&gt; container.
- * 
+ *
  * Return value: An XML string or NULL if no notation data is available.
  **/
 char *
-gpgme_get_notation ( GpgmeCtx c )
+gpgme_get_notation (GpgmeCtx ctx)
 {
-    if ( !c->notation )
-        return NULL;
-    return _gpgme_data_get_as_string ( c->notation );
+  if (!ctx->notation)
+    return NULL;
+  return _gpgme_data_get_as_string (ctx->notation);
 }
 
 
 /**
  * gpgme_get_op_info:
- * @c: the context 
- * @reserved: 
- * 
+ * @c: the context
+ * @reserved:
+ *
  * Return information about the last information.  The caller has to
  * free the string.  NULL is returned if there is not previous
  * operation available or the operation has not yet finished.
@@ -168,123 +160,212 @@ gpgme_get_notation ( GpgmeCtx c )
  * </GnupgOperationInfo>
  * ]]>
  * </literal>
- * Return value: NULL for no info available or an XML string 
+ * Return value: NULL for no info available or an XML string
  **/
 char *
-gpgme_get_op_info ( GpgmeCtx c, int reserved )
+gpgme_get_op_info (GpgmeCtx ctx, int reserved)
 {
-    if (!c || reserved)
-        return NULL; /*invalid value */
- 
-    return _gpgme_data_get_as_string (c->op_info);
+  if (!ctx || reserved)
+    return NULL;  /* Invalid value.  */
+
+  return _gpgme_data_get_as_string (ctx->op_info);
 }
 
+
 /*
  * Store the data object with the operation info in the
- * context. Caller should not use that object anymore.  
+ * context. Caller should not use that object anymore.
  */
 void
-_gpgme_set_op_info (GpgmeCtx c, GpgmeData info)
+_gpgme_set_op_info (GpgmeCtx ctx, GpgmeData info)
 {
-    assert (c);
+  assert (ctx);
 
-    gpgme_data_release (c->op_info); 
-    c->op_info = NULL;
+  gpgme_data_release (ctx->op_info);
+  ctx->op_info = NULL;
 
-    if (info)
-        c->op_info = info;
+  if (info)
+    ctx->op_info = info;
+}
+
+
+GpgmeError
+gpgme_set_protocol (GpgmeCtx ctx, GpgmeProtocol protocol)
+{
+  if (!ctx)
+    return mk_error (Invalid_Value);
+
+  switch (protocol)
+    {
+    case GPGME_PROTOCOL_OpenPGP:
+      ctx->use_cms = 0;
+      break;
+    case GPGME_PROTOCOL_CMS:
+      ctx->use_cms = 1;
+      break;
+    case GPGME_PROTOCOL_AUTO:
+      return mk_error (Not_Implemented);
+    default:
+      return mk_error (Invalid_Value);
+    }
+
+  return 0;
 }
 
 
 /**
  * gpgme_set_armor:
- * @c: the contect 
+ * @ctx: the context
  * @yes: boolean value to set or clear that flag
- * 
- * Enable or disable the use of an ascii armor for all output.  
+ *
+ * Enable or disable the use of an ascii armor for all output.
  **/
 void
-gpgme_set_armor ( GpgmeCtx c, int yes )
+gpgme_set_armor (GpgmeCtx ctx, int yes)
 {
-    if ( !c )
-        return; /* oops */
-    c->use_armor = yes;
+  if (!ctx)
+    return;
+  ctx->use_armor = yes;
 }
 
 
 /**
  * gpgme_get_armor:
- * @c: the context
- * 
+ * @ctx: the context
+ *
  * Return the state of the armor flag which can be changed using
  * gpgme_set_armor().
- * 
+ *
  * Return value: Boolean whether armor mode is to be used.
  **/
-int 
-gpgme_get_armor (GpgmeCtx c)
+int
+gpgme_get_armor (GpgmeCtx ctx)
 {
-    return c && c->use_armor;
+  return ctx && ctx->use_armor;
 }
 
 
 /**
  * gpgme_set_textmode:
- * @c: the context 
+ * @ctx: the context
  * @yes: boolean flag whether textmode should be enabled
- * 
+ *
  * Enable or disable the use of the special textmode.  Textmode is for example
- * used for MIME (RFC2015) signatures
+ * used for the RFC2015 signatures; note that the updated RFC 3156 mandates
+ * that the MUA does some preparations so that textmode is not needed anymore.
  **/
 void
-gpgme_set_textmode ( GpgmeCtx c, int yes )
+gpgme_set_textmode (GpgmeCtx ctx, int yes)
 {
-    if ( !c )
-        return; /* oops */
-    c->use_textmode = yes;
+  if (!ctx)
+    return;
+  ctx->use_textmode = yes;
 }
 
 /**
  * gpgme_get_textmode:
- * @c: the context
- * 
+ * @ctx: the context
+ *
  * Return the state of the textmode flag which can be changed using
  * gpgme_set_textmode().
- * 
+ *
  * Return value: Boolean whether textmode is to be used.
  **/
-int 
-gpgme_get_textmode (GpgmeCtx c)
+int
+gpgme_get_textmode (GpgmeCtx ctx)
 {
-    return c && c->use_textmode;
+  return ctx && ctx->use_textmode;
 }
 
 
+/**
+ * gpgme_set_include_certs:
+ * @ctx: the context
+ *
+ * Set the number of certifications to include in an S/MIME message.
+ * The default is 1 (only the cert of the sender).  -1 means all certs,
+ * and -2 means all certs except the root cert.
+ *
+ * Return value: Boolean whether textmode is to be used.
+ **/
+void
+gpgme_set_include_certs (GpgmeCtx ctx, int nr_of_certs)
+{
+  if (nr_of_certs < -2)
+    ctx->include_certs = -2;
+  else
+    ctx->include_certs = nr_of_certs;
+}
+
+
+/**
+ * gpgme_get_include_certs:
+ * @ctx: the context
+ *
+ * Get the number of certifications to include in an S/MIME message.
+ *
+ * Return value: Boolean whether textmode is to be used.
+ **/
+int
+gpgme_get_include_certs (GpgmeCtx ctx)
+{
+  return ctx->include_certs;
+}
+
 
 /**
  * gpgme_set_keylist_mode:
- * @c: the context
+ * @ctx: the context
  * @mode: listing mode
- * 
- * This function changes the default behaviour of the keylisting functions.
+ *
+ * This function changes the default behaviour of the keylisting
+ * functions.  mode is a bitwise-OR of the GPGME_KEYLIST_* flags.
+ * The default mode is GPGME_KEYLIST_MODE_LOCAL.
+ *
+ * Return value: GPGME_Invalid_Value if ctx is not a context or mode
+ * not a valid mode.
+ **/
+GpgmeError
+gpgme_set_keylist_mode (GpgmeCtx ctx, int mode)
+{
+  if (!ctx)
+    return mk_error (Invalid_Value);
+
+  if (!((mode & GPGME_KEYLIST_MODE_LOCAL)
+	|| (mode & GPGME_KEYLIST_MODE_EXTERN)))
+    return mk_error (Invalid_Value);
+
+  ctx->keylist_mode = mode;
+  return 0;
+}
+
+
+/**
+ * gpgme_get_keylist_mode:
+ * @ctx: the context
+ *
+ * This function ch the default behaviour of the keylisting functions.
  * Defines values for @mode are: %0 = normal, %1 = fast listing without
  * information about key validity.
+ *
+ * Return value: 0 if ctx is not a valid context, or the current mode.
+ * Note that 0 is never a valid mode.
  **/
-void
-gpgme_set_keylist_mode ( GpgmeCtx c, int mode )
+int
+gpgme_get_keylist_mode (GpgmeCtx ctx)
 {
-    if (!c)
-        return;
-    c->keylist_mode = mode;
+  if (!ctx)
+    return 0;
+  return ctx->keylist_mode;
 }
 
 
 /**
  * gpgme_set_passphrase_cb:
- * @c: the context 
+ * @ctx: the context
  * @cb: A callback function
  * @cb_value: The value passed to the callback function
- * 
+ *
  * This function sets a callback function to be used to pass a passphrase
  * to gpg. The preferred way to handle this is by using the gpg-agent, but
  * because that beast is not ready for real use, you can use this passphrase
@@ -294,11 +375,11 @@ gpgme_set_keylist_mode ( GpgmeCtx c, int mode )
  * <literal>
  * typedef const char *(*GpgmePassphraseCb)(void*cb_value,
  *                                          const char *desc,
- *                                          void *r_hd);
+ *                                          void **r_hd);
  * </literal>
  * and called whenever gpgme needs a passphrase. DESC will have a nice
  * text, to be used to prompt for the passphrase and R_HD is just a parameter
- * to be used by the callback it self.  Becuase the callback returns a const
+ * to be used by the callback it self.  Because the callback returns a const
  * string, the callback might want to know when it can release resources
  * assocated with that returned string; gpgme helps here by calling this
  * passphrase callback with an DESC of %NULL as soon as it does not need
@@ -307,23 +388,26 @@ gpgme_set_keylist_mode ( GpgmeCtx c, int mode )
  *
  **/
 void
-gpgme_set_passphrase_cb ( GpgmeCtx c, GpgmePassphraseCb cb, void *cb_value )
+gpgme_set_passphrase_cb (GpgmeCtx ctx, GpgmePassphraseCb cb, void *cb_value)
 {
-    c->passphrase_cb = cb;
-    c->passphrase_cb_value = cb_value;
+  if (ctx)
+    {
+      ctx->passphrase_cb = cb;
+      ctx->passphrase_cb_value = cb_value;
+    }
 }
 
 /**
- * gpgme_set_pprogress_cb:
- * @c: the context 
+ * gpgme_set_progress_cb:
+ * @ctx: the context
  * @cb: A callback function
  * @cb_value: The value passed to the callback function
- * 
+ *
  * This function sets a callback function to be used as a progress indicator.
  *
  * The callback function is defined as:
  * <literal>
- * typedef void (*GpgmeProgressCb) (void*cb_value,
+ * typedef void (*GpgmeProgressCb) (void *cb_value,
  *                                  const char *what, int type,
  *                                  int curretn, int total);
  * </literal>
@@ -331,17 +415,11 @@ gpgme_set_passphrase_cb ( GpgmeCtx c, GpgmePassphraseCb cb, void *cb_value )
  * status in the file doc/DETAILS of the GnuPG distribution.
  **/
 void
-gpgme_set_progress_cb ( GpgmeCtx c, GpgmeProgressCb cb, void *cb_value )
+gpgme_set_progress_cb (GpgmeCtx ctx, GpgmeProgressCb cb, void *cb_value)
 {
-    c->progress_cb = cb;
-    c->progress_cb_value = cb_value;
+  if (ctx)
+    {
+      ctx->progress_cb = cb;
+      ctx->progress_cb_value = cb_value;
+    }
 }
-
-
-
-
-
-
-
-
-
