@@ -1,6 +1,6 @@
 /* ops.h - internal operations stuff 
  *	Copyright (C) 2000 Werner Koch (dd9jn)
- *      Copyright (C) 2001 g10 Code GmbH
+ *      Copyright (C) 2001, 2002 g10 Code GmbH
  *
  * This file is part of GPGME.
  *
@@ -23,6 +23,24 @@
 #define OPS_H
 
 #include "types.h"
+#include "rungpg.h"
+
+/* Support macros.  */
+
+#define test_and_allocate_result(ctx,field) \
+  do \
+    { \
+      if (!ctx->result.field) \
+        { \
+          ctx->result.field = xtrycalloc (1, sizeof *ctx->result.field); \
+          if (!ctx->result.field) \
+            { \
+              ctx->error = mk_error (Out_Of_Core); \
+              return; \
+            } \
+        } \
+    } \
+  while (0)
 
 /*-- gpgme.c --*/
 void _gpgme_release_result ( GpgmeCtx c );
@@ -36,9 +54,6 @@ void _gpgme_thaw_fd ( int fd );
 
 
 /*-- recipient.c --*/
-void _gpgme_append_gpg_args_from_recipients (
-    const GpgmeRecipients rset,
-    GpgObject gpg );
 int _gpgme_recipients_all_valid ( const GpgmeRecipients rset );
 
 
@@ -61,6 +76,9 @@ GpgmeError    _gpgme_data_append_percentstring_for_xml ( GpgmeData dh,
 GpgmeError    _gpgme_data_unread (GpgmeData dh,
                                   const char *buffer, size_t length );
 
+int _gpgme_data_inbound_handler (void *opaque, int pid, int fd);
+int _gpgme_data_outbound_handler (void *opaque, int pid, int fd);
+
 
 /*-- key.c --*/
 GpgmeError _gpgme_key_new ( GpgmeKey *r_key );
@@ -68,21 +86,52 @@ GpgmeError _gpgme_key_new_secret ( GpgmeKey *r_key );
 
 
 /*-- verify.c --*/
-void _gpgme_release_verify_result ( VerifyResult res );
+void _gpgme_release_verify_result (VerifyResult result);
+GpgmeSigStat _gpgme_intersect_stati (VerifyResult result);
+void _gpgme_verify_status_handler (GpgmeCtx ctx, GpgStatusCode code,
+				   char *args);
 
 /*-- decrypt.c --*/
-void _gpgme_release_decrypt_result ( DecryptResult res );
+void _gpgme_release_decrypt_result (DecryptResult result);
+void _gpgme_decrypt_status_handler (GpgmeCtx ctx, GpgStatusCode code,
+				    char *args);
+GpgmeError _gpgme_decrypt_start (GpgmeCtx ctx, GpgmeData ciph, GpgmeData plain,
+				 void *status_handler);
+GpgmeError _gpgme_decrypt_result (GpgmeCtx ctx);
 
 /*-- sign.c --*/
 void _gpgme_release_sign_result ( SignResult res );
+void _gpgme_sign_status_handler (GpgmeCtx ctx, GpgStatusCode code,
+				 char *args);
 
 /*-- encrypt.c --*/
 void _gpgme_release_encrypt_result ( EncryptResult res );
+void _gpgme_encrypt_status_handler (GpgmeCtx ctx, GpgStatusCode code,
+				    char *args);
+
+/*-- passphrase.c --*/
+void _gpgme_release_passphrase_result (PassphraseResult result);
+void _gpgme_passphrase_status_handler (GpgmeCtx ctx, GpgStatusCode code,
+				       char *args);
+GpgmeError _gpgme_passphrase_start (GpgmeCtx ctx);
+
+/*-- progress.c --*/
+void _gpgme_progress_status_handler (GpgmeCtx ctx, GpgStatusCode code,
+				     char *args);
+
+/*-- import.c --*/
+void _gpgme_release_import_result (ImportResult res);
+
+/*-- delete.c --*/
+void _gpgme_release_delete_result (DeleteResult res);
+
+/*-- genkey.c --*/
+void _gpgme_release_genkey_result (GenKeyResult res);
+
+/*-- version.c --*/
+const char *_gpgme_compare_versions (const char *my_version,
+				     const char *req_version);
+char *_gpgme_get_program_version (const char *const path);
 
 
 #endif /* OPS_H */
-
-
-
-
-
