@@ -48,7 +48,6 @@ typedef enum  {
     STATUS_SHM_GET_BOOL       ,
     STATUS_SHM_GET_HIDDEN     ,
     STATUS_NEED_PASSPHRASE    ,
-    STATUS_USERID_HINT        ,
     STATUS_UNEXPECTED         ,
     STATUS_VALIDSIG           ,
     STATUS_SIG_ID	      ,
@@ -81,9 +80,11 @@ typedef enum  {
     STATUS_GOT_IT	      ,
     STATUS_PROGRESS 	      ,
     STATUS_SIG_CREATED	      ,
+    STATUS_KEY_CREATED	      ,
     STATUS_SESSION_KEY        ,
     STATUS_NOTATION_NAME      ,
     STATUS_NOTATION_DATA      ,
+    STATUS_USERID_HINT	      ,
     STATUS_POLICY_URL         ,
     STATUS_BEGIN_STREAM       ,
     STATUS_END_STREAM         ,
@@ -96,6 +97,8 @@ typedef void (*GpgColonLineHandler)( GpgmeCtx, char *line );
 typedef const char *(*GpgCommandHandler)(void*, GpgStatusCode code,
                                          const char *keyword);
 
+const char *_gpgme_gpg_get_version (void);
+GpgmeError _gpgme_gpg_check_version (void);
 
 GpgmeError _gpgme_gpg_new ( GpgObject *r_gpg );
 void       _gpgme_gpg_release ( GpgObject gpg );
@@ -117,14 +120,31 @@ GpgmeError _gpgme_gpg_set_command_handler ( GpgObject gpg,
                                             GpgCommandHandler fnc,
                                             void *fnc_value );
 
-GpgmeError _gpgme_gpg_spawn ( GpgObject gpg, void *opaque );
-
-
+GpgmeError _gpgme_gpg_op_decrypt (GpgObject gpg, GpgmeData ciph,
+				  GpgmeData plain);
+GpgmeError _gpgme_gpg_op_delete (GpgObject gpg, GpgmeKey key, int allow_secret);
+GpgmeError _gpgme_gpg_op_encrypt (GpgObject gpg, GpgmeRecipients recp,
+				  GpgmeData plain, GpgmeData ciph,
+				  int use_armor);
+GpgmeError _gpgme_gpg_op_encrypt_sign (GpgObject gpg, GpgmeRecipients recp,
+				       GpgmeData plain, GpgmeData ciph,
+				       int use_armor, GpgmeCtx ctx);
+GpgmeError _gpgme_gpg_op_export (GpgObject gpg, GpgmeRecipients recp,
+				 GpgmeData keydata, int use_armor);
+GpgmeError _gpgme_gpg_op_genkey (GpgObject gpg, GpgmeData help_data,
+				 int use_armor, GpgmeData pubkey,
+				 GpgmeData seckey);
+GpgmeError _gpgme_gpg_op_import (GpgObject gpg, GpgmeData keydata);
+GpgmeError _gpgme_gpg_op_keylist (GpgObject gpg, const char *pattern,
+				  int secret_only, int keylist_mode);
+GpgmeError _gpgme_gpg_op_keylist_ext (GpgObject gpg, const char *pattern[],
+				      int secret_only, int reserved,
+				      int keylist_mode);
+GpgmeError _gpgme_gpg_op_sign (GpgObject gpg, GpgmeData in, GpgmeData out,
+			       GpgmeSigMode mode, int use_armor,
+			       int use_textmode, GpgmeCtx ctx /* FIXME */);
+GpgmeError _gpgme_gpg_op_trustlist (GpgObject gpg, const char *pattern);
+GpgmeError _gpgme_gpg_op_verify (GpgObject gpg, GpgmeData sig, GpgmeData text);
+GpgmeError _gpgme_gpg_spawn (GpgObject gpg, void *opaque);
 
 #endif /* RUNGPG_H */
-
-
-
-
-
-
