@@ -1,6 +1,6 @@
 /* types.h -  Some type definitions
  *	Copyright (C) 2000 Werner Koch (dd9jn)
- *      Copyright (C) 2001 g10 Code GmbH
+ *      Copyright (C) 2001, 2002 g10 Code GmbH
  *
  * This file is part of GPGME.
  *
@@ -44,11 +44,18 @@ typedef enum {
  * Declaration of internal objects
  */
 
+/*-- engine.c --*/
+struct engine_object_s;
+typedef struct engine_object_s *EngineObject;
+
+/*-- engine-gpgsm.c --*/
+struct gpgsm_object_s;
+typedef struct gpgsm_object_s *GpgsmObject;
+
 /*-- rungpg.c --*/
 struct gpg_object_s;
 typedef struct gpg_object_s *GpgObject;
 
-
 /*-- verify.c --*/
 struct verify_result_s;
 typedef struct verify_result_s *VerifyResult;
@@ -65,12 +72,21 @@ typedef struct sign_result_s *SignResult;
 struct encrypt_result_s;
 typedef struct encrypt_result_s *EncryptResult;
 
-/*-- key.c --*/
+/*-- passphrase.c --*/
+struct passphrase_result_s;
+typedef struct passphrase_result_s *PassphraseResult;
+
+/*-- import.c --*/
+struct import_result_s;
+typedef struct import_result_s *ImportResult;
+
+/*-- delete.c --*/
+struct delete_result_s;
+typedef struct delete_result_s *DeleteResult;
+
+/*-- genkey.c --*/
+struct genkey_result_s;
+typedef struct genkey_result_s *GenKeyResult;
 
 
 #endif /* TYPES_H */
-
-
-
-
-
