@@ -134,12 +134,13 @@ int asprintf (char **result, const char *format, ...);
 #endif
 
 #if !HAVE_FOPENCOOKIE
+#include <fcntl.h> /* make sure that ssize_t and off_t are defined */
 typedef struct
 {
   ssize_t (*read)(void*,char*,size_t);
   ssize_t (*write)(void*,const char*,size_t);
   int (*seek)(void*,off_t*,int);
-  int (*close)(coid*);
+  int (*close)(void*);
 } _IO_cookie_io_functions_t;
 typedef _IO_cookie_io_functions_t cookie_io_functions_t;
 FILE *fopencookie (void *cookie, const char *opentype,
