@@ -1,6 +1,6 @@
 /* -*- Mode: C -*-
 
-  $Id: cryptplug.h,v 1.11 2002/03/01 17:54:26 kalle Exp $
+  $Id: cryptplug.h,v 1.17 2002/03/22 10:40:24 khz Exp $
 
   CRYPTPLUG - an independent cryptography plug-in API
 
@@ -161,14 +161,25 @@ typedef char bool;
 */
 
 
-// dummy values:
 typedef enum {
-  CryptPlugFeat_undef             = 0,
+  Feature_undef             = 0,
 
-  CryptPlugFeat_SignMessages      = 1,
-  CryptPlugFeat_VerifySignatures  = 2,
-  CryptPlugFeat_EncryptMessages   = 3,
-  CryptPlugFeat_DecryptMessages   = 4   // more to follow ...
+  Feature_SignMessages      = 1,
+  Feature_VerifySignatures  = 2,
+  Feature_EncryptMessages   = 3,
+  Feature_DecryptMessages   = 4,
+  Feature_SendCertificates  = 5,
+  Feature_WarnSignCertificateExpiry = 6,
+  Feature_WarnSignEmailNotInCertificate = 7,
+  Feature_PinEntrySettings  = 8,
+  Feature_StoreMessagesWithSigs = 9,
+  Feature_EncryptionCRLs    = 10,
+  Feature_WarnEncryptCertificateExpiry = 11,
+  Feature_WarnEncryptEmailNotInCertificate = 12,
+  Feature_StoreMessagesEncrypted = 13,
+  Feature_CheckCertificatePath = 14,
+  Feature_CertificateDirectoryService = 15,
+  Feature_CRLDirectoryService = 16
 } Feature;
 
 // dummy values
@@ -427,7 +438,7 @@ SignEmail signEmail( void );
 */
 void setWarnSendUnsigned( bool );    
 
-    
+
 /*! \ingroup groupConfigSign
   \brief Returns whether a warning should be emitted when the user
   tries to send an email message unsigned.
@@ -769,6 +780,14 @@ void setReceiverCertificateExpiryNearWarning( bool );
   certificate of the receiver expires in the near future.
 */
 bool receiverCertificateExpiryNearWarning( void );
+
+
+/*! \ingroup groupConfigCrypt
+  \brief Returns the number of days until the specified receiver
+  certificate expires.
+*/
+int receiverCertificateDaysLeftToExpiry( const char* certificate );
+
     
 
 /*! \ingroup groupConfigCrypt
@@ -810,6 +829,13 @@ void setCertificateInChainExpiryNearWarningInterval( int );
 */
 int certificateInChainExpiryNearWarningInterval( void );
 
+
+/*! \ingroup groupConfigCrypt
+  \brief Returns the number of days until the first certificate in
+  the chain of the receiver certificate expires.
+*/
+int certificateInChainDaysLeftToExpiry( const char* certificate );
+
     
 /*! \ingroup groupConfigCrypt
   \brief Specifies whether a warning is emitted if the email address
@@ -1360,7 +1386,12 @@ struct StructuringInfo {
 /*! \ingroup groupSignAct
    \brief Signs a message \c cleartext and returns
           in \c *ciphertext the signature data bloc that
-          is to be added to the message.
+          is to be added to the message. The length returned
+          in \c *cipherLen tells you the size (==amount of bytes)
+          of the ciphertext, if the structuring information
+          would return with contentTEncCode set to "base64"
+          the ciphertext might contain a char 0x00
+          and has to be converted into base64 before sending.
 
    The signature role is specified by \c certificate.
    If \c certificate is \c NULL, the default certificate is used.
@@ -1390,6 +1421,7 @@ struct StructuringInfo {
 */
 bool signMessage( const char*  cleartext,
                   const char** ciphertext,
+                  const size_t* cipherLen,
                   const char*  certificate,
                   struct StructuringInfo* structuring );
 
@@ -1427,6 +1459,8 @@ struct SignatureMetaData {
 */
 bool checkMessageSignature( const char* ciphertext,
                             const char* signaturetext,
+                            bool signatureIsBinary,
+                            int signatureLen,
                             struct SignatureMetaData* sigmeta );
 
 /*! \ingroup groupSignAct
@@ -1436,11 +1470,25 @@ bool checkMessageSignature( const char* ciphertext,
 bool storeCertificatesFromMessage( const char* ciphertext );
 
 
+/*! \ingroup groupCryptAct
+   \brief Find all certificate for a given addressee.
+
+   NOTE: The \c certificate parameter must point to an allready allocated
+   block of memory which is large enough to hold the complete list.
+*/
+bool findCertificates( const char* addressee, char** certificates );
+
 /*! \ingroup groupCryptAct
    \brief Encrypts an email message in
           \c cleartext according to the \c addressee and
           the current settings (algorithm, etc.) and
           returns the encoded data bloc in \c *ciphertext.
+          The length returned in \c *cipherLen tells you the
+          size (==amount of bytes) of the ciphertext, if the
+          structuring information would return with
+          contentTEncCode set to "base64" the ciphertext
+          might contain a char 0x00 and has to be converted
+          into base64 before sending.
 
    If the message could be encrypted, the function returns
           \c true, otherwise
@@ -1465,6 +1513,7 @@ bool storeCertificatesFromMessage( const char* ciphertext );
 */
 bool encryptMessage( const char*  cleartext,
                      const char** ciphertext,
+                     const size_t* cipherLen,
                      const char*  addressee,
                      struct StructuringInfo* structuring );
 
@@ -1513,8 +1562,11 @@ bool encryptAndSignMessage( const char* cleartext,
           \c true, otherwise
           \c false.
 */
-bool decryptMessage( const char* ciphertext, const
-          char** cleartext, const char* certificate );
+bool decryptMessage( const char* ciphertext,
+                     bool        cipherIsBinary,
+                     int         cipherLen,
+                     const char** cleartext,
+                     const char* certificate );
 
 /*! \ingroup groupCryptAct
    \brief Combines the functionality of
