@@ -1,6 +1,6 @@
 /* -*- Mode: C -*-
 
-  $Id: gpgmeplug.c,v 1.38 2002/03/01 17:54:26 kalle Exp $
+  $Id: gpgmeplug.c,v 1.51 2002/03/23 11:35:31 werner Exp $
 
   GPGMEPLUG - an GPGME based cryptography plug-in following
               the common CRYPTPLUG specification.
@@ -50,6 +50,7 @@
 #include <assert.h>
 #include <errno.h>
 #include <time.h>
+#include <ctype.h>
 
 #ifndef BUG_URL
 #define BUG_URL "http:://www.gnupg.org/aegypten/"
@@ -60,7 +61,7 @@
 #define GPGMEPLUG_PROTOCOL GPGME_PROTOCOL_OpenPGP
 #endif
 
-// definitions for signing
+/* definitions for signing */
 #ifndef GPGMEPLUG_SIGN_MAKE_MIME_OBJECT
 #define GPGMEPLUG_SIGN_INCLUDE_CLEARTEXT true
 #define GPGMEPLUG_SIGN_MAKE_MIME_OBJECT  true
@@ -78,8 +79,9 @@
 #define GPGMEPLUG_SIGN_FLAT_PREFIX       ""
 #define GPGMEPLUG_SIGN_FLAT_SEPARATOR    ""
 #define GPGMEPLUG_SIGN_FLAT_POSTFIX      ""
+#define __GPGMEPLUG_SIGNATURE_CODE_IS_BINARY false
 #endif
-// definitions for encoding
+/* definitions for encoding */
 #ifndef GPGMEPLUG_ENC_MAKE_MIME_OBJECT
 #define GPGMEPLUG_ENC_INCLUDE_CLEARTEXT  false
 #define GPGMEPLUG_ENC_MAKE_MIME_OBJECT   true
@@ -97,12 +99,13 @@
 #define GPGMEPLUG_ENC_FLAT_PREFIX        ""
 #define GPGMEPLUG_ENC_FLAT_SEPARATOR     ""
 #define GPGMEPLUG_ENC_FLAT_POSTFIX       ""
+#define __GPGMEPLUG_ENCRYPTED_CODE_IS_BINARY false
 #endif
-// Note: The following specification will result in
-//       function encryptAndSignMessage() producing
-//       _empty_ mails.
-//       This must be changed as soon as our plugin
-//       is supporting the encryptAndSignMessage() function.
+/* Note: The following specification will result in
+       function encryptAndSignMessage() producing
+       _empty_ mails.
+       This must be changed as soon as our plugin
+       is supporting the encryptAndSignMessage() function. */
 #ifndef GPGMEPLUG_ENCSIGN_MAKE_MIME_OBJECT
 #define GPGMEPLUG_ENCSIGN_INCLUDE_CLEARTEXT false
 #define GPGMEPLUG_ENCSIGN_MAKE_MIME_OBJECT  false
@@ -227,14 +230,27 @@ void deinitialize()
 
 bool hasFeature( Feature flag )
 {
+  /* our own plugins are supposed to support everything */
   switch ( flag ) {
-    case CryptPlugFeat_SignMessages:              return true;
-    case CryptPlugFeat_VerifySignatures:          return true;
-    case CryptPlugFeat_EncryptMessages:           return true;
-    case CryptPlugFeat_DecryptMessages:           return true;
-    // undefined or not yet implemented:
-    case CryptPlugFeat_undef:                     return false;
-    default:                                      return false;
+  case Feature_SignMessages:              return true;
+  case Feature_VerifySignatures:          return true;
+  case Feature_EncryptMessages:           return true;
+  case Feature_DecryptMessages:           return true;
+  case Feature_SendCertificates:          return true;
+  case Feature_WarnSignCertificateExpiry: return true;
+  case Feature_WarnSignEmailNotInCertificate: return true;
+  case Feature_PinEntrySettings:          return true;
+  case Feature_StoreMessagesWithSigs:     return true;
+  case Feature_EncryptionCRLs:            return true;
+  case Feature_WarnEncryptCertificateExpiry: return true;
+  case Feature_WarnEncryptEmailNotInCertificate: return true;
+  case Feature_StoreMessagesEncrypted:    return true;
+  case Feature_CheckCertificatePath:      return true;
+  case Feature_CertificateDirectoryService: return true;
+  case Feature_CRLDirectoryService:       return true;
+  /* undefined or not yet implemented: */
+  case Feature_undef:                     return false;
+  default:                                      return false;
   }
 }
 
@@ -357,7 +373,7 @@ bool isEmailInCertificate( const char* email, const char* certificate )
        Another note: OK, OK, we'll handle that in the MUA. You can
        assume that you only get the email address.
     */
-    return false; // dummy
+  return false; /* dummy*/
 }
 
 
@@ -439,7 +455,7 @@ int signatureCertificateDaysLeftToExpiry( const char* certificate )
        Please return the number of days that are left until the
        certificate specified in the parameter certificate expires.
     */
-    return 10; // dummy that triggers a warning in the MUA
+  return 10; /* dummy that triggers a warning in the MUA */
 }
 
 
@@ -470,7 +486,7 @@ int caCertificateDaysLeftToExpiry( const char* certificate )
        CA certificate for the certificate specified in the parameter
        certificate expires.
     */
-    return 10; // dummy that triggers a warning in the MUA
+  return 10; /* dummy that triggers a warning in the MUA */
 }
 
 void setCACertificateExpiryNearInterval( int interval )
@@ -500,7 +516,7 @@ int rootCertificateDaysLeftToExpiry( const char* certificate )
        root certificate for the certificate specified in the parameter
        certificate expires.
     */
-    return 10; // dummy that triggers a warning in the MUA
+  return 10; /* dummy that triggers a warning in the MUA */
 }
 
 
@@ -631,6 +647,17 @@ bool receiverCertificateExpiryNearWarning()
   return config.receiverCertificateExpiryNearWarning;
 }
 
+
+int receiverCertificateDaysLeftToExpiry( const char* certificate )
+{
+    /* PENDING(g10)
+       Please return the number of days that are left until the
+       certificate specified in the parameter certificate expires.
+    */
+  return 10; /* dummy that triggers a warning in the MUA */
+}
+
+
 void setReceiverCertificateExpiryNearWarningInterval( int interval )
 {
   config.receiverCertificateExpiryNearWarningInterval = interval;
@@ -651,6 +678,18 @@ bool certificateInChainExpiryNearWarning()
   return config.certificateInChainExpiryNearWarning;
 }
 
+
+int certificateInChainDaysLeftToExpiry( const char* certificate )
+{
+    /* PENDING(g10)
+       Please return the number of days that are left until the
+       the first certificate in the chain of the specified certificate
+       expires.
+    */
+  return 10; /* dummy that triggers a warning in the MUA */
+}
+
+
 void setCertificateInChainExpiryNearWarningInterval( int interval )
 {
   config.certificateInChainExpiryNearWarningInterval = interval;
@@ -700,7 +739,7 @@ int encryptionCRLsDaysLeftToExpiry()
        Please return the number of days that are left until the
        CRL used for encryption expires.
     */
-    return 10; // dummy that triggers a warning in the MUA
+  return 10; /* dummy that triggers a warning in the MUA */
 }
 
 void setEncryptionCRLExpiryNearWarning( bool flag )
@@ -823,15 +862,15 @@ void storeNewCharPtr( char** dest, const char* src )
 
 bool signMessage( const char*  cleartext,
                   const char** ciphertext,
+                  const size_t* cipherLen,
                   const char*  certificate,
                   struct StructuringInfo* structuring )
 {
   GpgmeCtx ctx;
   GpgmeError err;
   GpgmeData data,  sig;
-  size_t rSLen = 0;
-  char*  rSig  = 0;
-  bool   bOk   = false;
+  char* rSig  = 0;
+  bool  bOk   = false;
   int sendCerts = 1;
 
   init_StructuringInfo( structuring );
@@ -842,8 +881,8 @@ bool signMessage( const char*  cleartext,
   err = gpgme_new (&ctx);
   gpgme_set_protocol (ctx, GPGMEPLUG_PROTOCOL);
 
-  gpgme_set_armor (ctx, 1);
-  gpgme_set_textmode (ctx, 1);
+  gpgme_set_armor (ctx, __GPGMEPLUG_SIGNATURE_CODE_IS_BINARY ? 0 : 1);
+  /*  gpgme_set_textmode (ctx, 1); */
 
   switch ( config.sendCertificates ) {
     case SendCert_undef:
@@ -865,32 +904,41 @@ bool signMessage( const char*  cleartext,
       break;
   }
   gpgme_set_include_certs (ctx, sendCerts);
-  
-  // PENDING(g10) Implement this
-  //gpgme_set_signature_algorithm( ctx, config.signatureAlgorithm );
+
+  /* PENDING(g10) Implement this 
+
+     gpgme_set_signature_algorithm( ctx, config.signatureAlgorithm )
+     --> This does not make sense.  The algorithm is a property of
+     the certificate used [wk 2002-03-23] */
 
   gpgme_data_new_from_mem (&data, cleartext,
-                            1+strlen( cleartext ), 1 );
+                            strlen( cleartext ), 1 );
   gpgme_data_new ( &sig );
   err = gpgme_op_sign (ctx, data, sig, GPGME_SIG_MODE_DETACH );
 
   if (!err) {
-    rSig  = gpgme_data_release_and_get_mem( sig,  &rSLen );
-    *ciphertext = malloc( rSLen + 1 );
-    if( *ciphertext ) {
-      if( rSLen ) {
-        bOk = true;
-        strncpy((char*)*ciphertext, rSig, rSLen );
+    if( __GPGMEPLUG_SIGNATURE_CODE_IS_BINARY ) {
+      *ciphertext = gpgme_data_release_and_get_mem( sig, (size_t*)cipherLen );
+      bOk = true;
+    }
+    else {
+      rSig = gpgme_data_release_and_get_mem( sig, (size_t*)cipherLen );
+      *ciphertext = malloc( *cipherLen + 1 );
+      if( *ciphertext ) {
+        if( *cipherLen ) {
+          bOk = true;
+          strncpy((char*)*ciphertext, rSig, *cipherLen );
+        }
+        ((char*)(*ciphertext))[*cipherLen] = '\0';
       }
-      ((char*)(*ciphertext))[rSLen] = '\0';
+      free( rSig );
     }
-    free( rSig );
   }
   else {
     gpgme_data_release( sig );
     *ciphertext = 0;
-    // hier fehlt eine Fehlerbehandlung, falls das
-    // Signieren schiefging
+    /* erro handling missing to detect wther signing failed (hier
+       fehlt eine Fehlerbehandlung, falls das Signieren schiefging) */
   }
   gpgme_data_release( data );
   gpgme_release (ctx);
@@ -973,6 +1021,8 @@ sig_status_to_string( GpgmeSigStat status )
 
 bool checkMessageSignature( const char* ciphertext,
                             const char* signaturetext,
+                            bool signatureIsBinary,
+                            int signatureLen,
                             struct SignatureMetaData* sigmeta )
 {
   GpgmeCtx ctx;
@@ -987,10 +1037,18 @@ bool checkMessageSignature( const char* ciphertext,
 
   gpgme_new( &ctx );
   gpgme_set_protocol (ctx, GPGMEPLUG_PROTOCOL);
+  gpgme_set_armor (ctx,    signatureIsBinary ? 0 : 1);
+  /*  gpgme_set_textmode (ctx, signatureIsBinary ? 0 : 1); */
+
   gpgme_data_new_from_mem( &datapart, ciphertext,
-                          1+strlen( ciphertext ), 1 );
-  gpgme_data_new_from_mem( &sigpart, signaturetext,
-                          1+strlen( signaturetext ), 1 );
+                          strlen( ciphertext ), 1 );
+
+  gpgme_data_new_from_mem( &sigpart,
+                           signaturetext,
+                           signatureIsBinary
+                           ? signatureLen
+                           : strlen( signaturetext ),
+                           1 );
 
   gpgme_op_verify( ctx, sigpart, datapart, &status );
   gpgme_data_release( datapart );
@@ -1004,9 +1062,9 @@ bool checkMessageSignature( const char* ciphertext,
     strcpy( sigmeta->status, statusStr );
     sigmeta->status[strlen( statusStr )] = '\0';
   } else
-    ; // nothing to do, is already 0
+    ; /* nothing to do, is already 0 */
 
-  // Extended information for any number of signatures.
+  /* Extended information for any number of signatures. */
   fpr = gpgme_get_sig_status( ctx, sig_idx, &status, &created );
   sigmeta->extended_info = 0;
   while( fpr != NULL ) {
@@ -1017,7 +1075,7 @@ bool checkMessageSignature( const char* ciphertext,
                                     sizeof( struct SignatureMetaDataExtendedInfo ) * ( sig_idx + 1 ) );
     if( realloc_return ) {
       sigmeta->extended_info = realloc_return;
-      // the creation time
+      /* the creation time */
       sigmeta->extended_info[sig_idx].creation_time = malloc( sizeof( struct tm ) );
       if( sigmeta->extended_info[sig_idx].creation_time ) {
         ctime_val = localtime( &created );
@@ -1040,8 +1098,8 @@ bool checkMessageSignature( const char* ciphertext,
         sigmeta->extended_info[sig_idx].fingerprint[strlen( fpr )] = '\0';
       }
     } else
-      break; // if allocation fails once, it isn't likely to
-              // succeed the next time either
+      break; /* if allocation fails once, it isn't likely to
+                succeed the next time either */
 
     fpr = gpgme_get_sig_status (ctx, ++sig_idx, &status, &created);
   }
@@ -1057,16 +1115,148 @@ bool storeCertificatesFromMessage(
         const char* ciphertext ){ return true; }
 
 
-bool encryptMessage( const char* cleartext,
+/* returns address if address doesn't contain a <xxx> part
+ * else it returns a new string xxx and frees address
+ */
+static char* parseAddress( char* address )
+{
+  char* result = address;
+  char* i;
+  char* j;
+  if( !result ) return result;
+  i = index( address, '<' );
+  if( i ) {
+    j = index( i+1, '>' );
+    if( j == NULL ) j = address+strlen(address);
+    result = malloc( j-i );
+    strncpy( result, i+1, j-i-1 );
+    result[j-i-1] = '\0';
+    free( address );
+  } else {
+    i = address;
+    j = i+strlen(address);
+  }
+  {
+    /* remove surrounding whitespace */
+    char* k = result+(j-i-1);
+    char* l = result;
+    while( isspace( *l ) ) ++l;
+    while( isspace( *k ) ) --k;
+    if( l != result || k != result+(j-i-1) ) {
+      char* result2 = malloc( k-l+2 );
+      strncpy( result2, l, k-l+1 );
+      result2[k-l+1] = '\0';
+      free(result);
+      result = result2;
+    }
+  }
+  return result;
+}
+
+static char* nextAddress( const char** address )
+{
+  const char *start = *address;
+  char* result = NULL;
+  int quote = 0;
+  int comment = 0;
+  int found = 0;
+  if( *address == NULL ) return NULL;
+  while( **address ) {
+
+    switch( **address ) {
+    case '\\': /* escaped character */
+      ++(*address);
+      break;
+    case '"':
+      if( comment == 0 ) {
+        if( quote > 0 ) --quote;
+        else ++quote;
+      }
+      break;
+    case '(': /* comment start */
+      if( quote == 0 ) ++comment;
+      break;
+    case ')': /* comment end */
+      if( quote == 0 ) --comment;
+      break;
+    case '\0':
+    case '\1': /* delimiter */
+      if( quote == 0 && comment == 0 ) {
+        found = 1;
+      }
+      break;
+    }
+    ++(*address);
+    if( found ) break;
+  }
+  if( found || **address == 0 ) {
+    size_t len;
+    len = *address - start;
+    if( len > 0 ) {
+      if( **address != 0 ) --len;
+      result = malloc( len*sizeof(char)+1 );
+      strncpy( result, start, len );
+      result[len] = '\0';
+    }
+  }
+  return parseAddress(result);
+}
+
+/*
+  Find all certificate for a given addressee and return them in a
+  '\1' separated list.
+  NOTE: The certificate parameter must point to an allready allocated
+  block of memory which is large enough to hold the complete list.
+*/
+bool findCertificates( const char* addressee, char** certificates )
+{
+  GpgmeCtx ctx;
+  GpgmeError err;
+  GpgmeKey rKey;
+  const char *s;
+  const char *s2;
+  int nFound = 0;
+
+  strcpy( *certificates, "" );
+
+  gpgme_new (&ctx);
+  gpgme_set_protocol (ctx, GPGMEPLUG_PROTOCOL);
+
+  err = gpgme_op_keylist_start(ctx, addressee, 0);
+  while( GPGME_No_Error == err ) {
+    err = gpgme_op_keylist_next(ctx, &rKey);
+    if( GPGME_No_Error == err ) {
+      s = gpgme_key_get_string_attr (rKey, GPGME_ATTR_USERID, NULL, 0);
+      if( s ) {
+        s2 = gpgme_key_get_string_attr (rKey, GPGME_ATTR_FPR, NULL, 0);
+        if( s2 ) {
+          if( nFound )
+            strcat(*certificates,"\1" );
+          strcat( *certificates, s );
+          strcat( *certificates, "    (" );
+          strcat( *certificates, s2 );
+          strcat( *certificates, ")" );
+          ++nFound;
+        }
+      }
+    }
+  }
+  gpgme_op_keylist_end( ctx );
+  gpgme_release (ctx);
+
+  return ( 0 < nFound );
+}
+
+bool encryptMessage( const char*  cleartext,
                      const char** ciphertext,
-                     const char* addressee,
+                     const size_t* cipherLen,
+                     const char*  certificate,
                      struct StructuringInfo* structuring )
 {
   GpgmeCtx ctx;
   GpgmeError err;
   GpgmeData gCiphertext, gPlaintext;
   GpgmeRecipients rset;
-  size_t rCLen = 0;
   char*  rCiph = 0;
   bool   bOk   = false;
 
@@ -1075,8 +1265,8 @@ bool encryptMessage( const char* cleartext,
   gpgme_new (&ctx);
   gpgme_set_protocol (ctx, GPGMEPLUG_PROTOCOL);
 
-  gpgme_set_armor (ctx, 1);
-  gpgme_set_textmode (ctx, 1);
+  gpgme_set_armor (ctx, __GPGMEPLUG_ENCRYPTED_CODE_IS_BINARY ? 0 : 1);
+  /*  gpgme_set_textmode (ctx, 1); */
 
   gpgme_data_new_from_mem (&gPlaintext, cleartext,
                             1+strlen( cleartext ), 1 );
@@ -1084,7 +1274,7 @@ bool encryptMessage( const char* cleartext,
 
   gpgme_recipients_new (&rset);
 
-
+  /*
   if( GPGMEPLUG_PROTOCOL == GPGME_PROTOCOL_CMS )
   {
     gpgme_recipients_add_name_with_validity (rset,
@@ -1093,11 +1283,43 @@ bool encryptMessage( const char* cleartext,
     fputs( "\nGPGSMPLUG encryptMessage() using test key of Aegypten Project\n", stderr );
   }
   else
+  */
   {
-    gpgme_recipients_add_name (rset, addressee);
-    fprintf( stderr, "\nGPGMEPLUG encryptMessage() using addressee %s\n", addressee );
+    const char* p = certificate;
+    char* tok;
+    while( (tok = nextAddress( &p ) ) != 0 ) {
+      if( GPGMEPLUG_PROTOCOL == GPGME_PROTOCOL_CMS )
+        gpgme_recipients_add_name_with_validity (rset, tok, GPGME_VALIDITY_FULL );
+      else
+        gpgme_recipients_add_name (rset, tok);
+      fprintf( stderr, "\nGPGMEPLUG encryptMessage() using addressee %s\n", tok );
+      free(tok);
+    }
   }
 
+  /* PENDING(g10) Implement this
+     Possible values: RSA = 1, SHA1 = 2, TripleDES = 3
+     gpgme_set_encryption_algorithm( ctx, config.encryptionAlgorithm );
+
+     -> Your are mixing public key and symmetric algorithms.  The
+     latter may be configured but the sphix specifications do opnly
+     allow 3-DES so this is not nothing we need to do.  The proper way
+     to select the symmetric algorithm is anyway by looking at the
+     capabilities of the certificate because this is the only way to
+     know what the recipient can accept. [wk 2002-03-23]
+
+     PENDING(g10) Implement this
+     gpgme_set_encryption_check_certificate_path(
+     config.checkCertificatePath )
+
+     PENDING(g10) Implement this
+     gpgme_set_encryption_check_certificate_path_to_root(
+     config.checkEncryptionCertificatePathToRoot )
+
+     -> Not checking a certificate up to the ROOT CA is dangerous and
+     stupid. There is no need for those options.  [wk 2002-03-23] */
+
+
 
   err = gpgme_op_encrypt (ctx, rset, gPlaintext, gCiphertext );
   if( err )
@@ -1107,23 +1329,31 @@ bool encryptMessage( const char* cleartext,
   gpgme_data_release (gPlaintext);
 
   if( !err ) {
-    rCiph = gpgme_data_release_and_get_mem( gCiphertext,  &rCLen );
-    *ciphertext = malloc( rCLen + 1 );
-    if( *ciphertext ) {
-      if( rCLen ) {
-        bOk = true;
-        strncpy((char*)*ciphertext, rCiph, rCLen );
+    if( __GPGMEPLUG_ENCRYPTED_CODE_IS_BINARY ) {
+      *ciphertext = gpgme_data_release_and_get_mem( gCiphertext, (size_t*)cipherLen );
+      bOk = true;
+    }
+    else {
+      rCiph = gpgme_data_release_and_get_mem( gCiphertext, (size_t*)cipherLen );
+      *ciphertext = malloc( *cipherLen + 1 );
+      if( *ciphertext ) {
+        if( *cipherLen ) {
+          bOk = true;
+          strncpy((char*)*ciphertext, rCiph, *cipherLen );
+        }
+        ((char*)(*ciphertext))[*cipherLen] = 0;
       }
-      ((char*)(*ciphertext))[rCLen] = 0;
+      free( rCiph );
     }
-    free( rCiph );
   }
   else {
     gpgme_data_release ( gCiphertext );
     *ciphertext = 0;
-    // hier fehlt eine Fehlerbehandlung: fuer einen Recipient nur ein
-    // untrusted key (oder gar keiner) gefunden wurde, verweigert gpg
-    // das signieren.
+    /* error handling is missing: if only one untrusted key was found
+      (or none at all), gpg won't sign the message.  (hier fehlt eine
+      Fehlerbehandlung: fuer einen Recipient nur ein untrusted key
+      (oder gar keiner) gefunden wurde, verweigert gpg das signieren.)
+    */
   }
 
   gpgme_release (ctx);
@@ -1181,7 +1411,7 @@ bool encryptAndSignMessage( const char* cleartext,
 
   bOk = false;
 
-  // implementation of this function is still missing
+  /* implementation of this function is still missing */
 
   if( bOk && structuring ) {
     structuring->includeCleartext = GPGMEPLUG_ENCSIGN_INCLUDE_CLEARTEXT;
@@ -1222,7 +1452,10 @@ bool encryptAndSignMessage( const char* cleartext,
   return bOk;
 }
 
+
 bool decryptMessage( const char* ciphertext,
+                     bool        cipherIsBinary,
+                     int         cipherLen,
                      const char** cleartext,
                      const char* certificate )
 {
@@ -1238,9 +1471,20 @@ bool decryptMessage( const char* ciphertext,
 
   err = gpgme_new (&ctx);
   gpgme_set_protocol (ctx, GPGMEPLUG_PROTOCOL);
+  
+  gpgme_set_armor (ctx, cipherIsBinary ? 0 : 1);
+  /*  gpgme_set_textmode (ctx, cipherIsBinary ? 0 : 1); */
 
+  /*
   gpgme_data_new_from_mem( &gCiphertext, ciphertext,
-                           1+strlen( ciphertext ), 1 );
+                           1+strlen( ciphertext ), 1 ); */
+  gpgme_data_new_from_mem( &gCiphertext,
+                           ciphertext,
+                           cipherIsBinary
+                           ? cipherLen
+                           : strlen( ciphertext ),
+                           1 );
+
   gpgme_data_new( &gPlaintext );
 
   gpgme_op_decrypt( ctx, gCiphertext, gPlaintext );
